package com.david.oviedotourist.activities;

import android.Manifest;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.pm.PackageManager;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.preference.PreferenceManager;
import android.provider.Settings;
import android.support.design.widget.NavigationView;
import android.support.v4.content.ContextCompat;
import android.support.v4.view.GravityCompat;
import android.support.v4.widget.DrawerLayout;
import android.support.v7.app.ActionBar;
import android.support.v7.app.AlertDialog;
import android.os.Bundle;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.support.v7.widget.Toolbar;
import android.view.MenuItem;
import android.view.View;
import android.widget.ProgressBar;
import android.widget.TextView;

import com.david.oviedotourist.R;
import com.david.oviedotourist.adapters.EventAdapter;
import com.david.oviedotourist.pojo.Site;
import com.david.oviedotourist.pojo.SiteType;
import com.david.oviedotourist.utilities.MyNotifications;
import com.david.oviedotourist.utilities.MyPermissions;
import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;
import com.google.firebase.database.ValueEventListener;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.Set;

public class MainActivity extends AppCompatActivity
        implements NavigationView.OnNavigationItemSelectedListener {

    private static final int ACCESS_FINE_LOCATION_REQUEST_CODE = 1234;
    private static final String INTERNET_ACTIVATION = Settings.ACTION_SETTINGS;
    private static final String FAVOURITE_LIST = "favourite";
    private static final String TO_VISIT_LIST = "visit";

    ///// FIREBASE DATABASE /////
    private static final String EVENTS_CHILD = "events/";
    private DatabaseReference databaseReference;
    private List<Site> eventList;
    private ValueEventListener eventListener;

    ///// RECYCLER VIEW /////
    private RecyclerView recyclerView;
    private RecyclerView.LayoutManager layoutManager;
    private EventAdapter eventAdapter;

    private DrawerLayout drawer;
    private SharedPreferences preferences;

    private ProgressBar progressBar;
    private TextView loadingText;

    //////////////////////////////Activity Lifecycle///////////////////////////////

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        Toolbar toolbar = (Toolbar) findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);

        if (getSupportActionBar() != null) {
            ActionBar actionBar = getSupportActionBar();
            actionBar.setDisplayHomeAsUpEnabled(true);
            actionBar.setHomeAsUpIndicator(R.drawable.ic_menu);
            actionBar.setTitle(getString(R.string.schedule));
        }

        checkInternetConnection();

        drawer = (DrawerLayout) findViewById(R.id.drawer_layout);
        NavigationView navigationView = (NavigationView) findViewById(R.id.nav_view);
        navigationView.setNavigationItemSelectedListener(this);

        progressBar = (ProgressBar) findViewById(R.id.progressBar);
        loadingText = (TextView) findViewById(R.id.tv_loading);

        preferences = PreferenceManager.getDefaultSharedPreferences(this);

        eventList = new ArrayList<>();
        databaseReference = FirebaseDatabase.getInstance()
                .getReference().child(EVENTS_CHILD);

        recyclerView = (RecyclerView) findViewById(R.id.image_content);
        eventAdapter = new EventAdapter(MainActivity.this, eventList);
        layoutManager = new LinearLayoutManager(MainActivity.this,
                LinearLayoutManager.VERTICAL, false);
        recyclerView.setAdapter(eventAdapter);
        recyclerView.setLayoutManager(layoutManager);

        eventListener = databaseReference.addValueEventListener(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {
                if (dataSnapshot.exists()) {
                    Iterable<DataSnapshot> references = dataSnapshot.getChildren();
                    while (references.iterator().hasNext()) {
                        DataSnapshot nextDataReference = references.iterator().next();
                        Site site = nextDataReference.getValue(Site.class);

                        if (site.getEvent().getDate() > System.currentTimeMillis()
                                && checkSiteType(site.getType())) {
                            eventList.add(site);
                        }
                        Collections.sort(eventList, new Comparator<Site>() {
                            @Override
                            public int compare(Site s1, Site s2) {
                                Long s1Date = s1.getEvent().getDate();
                                Long s2Date = s2.getEvent().getDate();
                                return s1Date.compareTo(s2Date);
                            }
                        });
                    }
                }

                progressBar.setVisibility(ProgressBar.INVISIBLE);
                loadingText.setVisibility(TextView.INVISIBLE);

                if (eventList.size() > 0) {
                    onClickListener();
                    eventAdapter.swap(eventList);
                } else {
                    MyNotifications.createToastNotification(MainActivity.this,
                            R.string.no_event, MyNotifications.LONG_DURATION);
                }
            }

            @Override
            public void onCancelled(DatabaseError databaseError) {
                MyNotifications.createToastNotification(MainActivity.this,
                        R.string.server_conexion_cancelled, MyNotifications.SHORT_DURATION);
            }
        });
    }

    @Override
    protected void onRestart() {
        super.onRestart();
        checkInternetConnection();
        checkNewEventsFromNetwork();
    }

    @Override
    protected void onDestroy() {
        databaseReference.removeEventListener(eventListener);
        super.onDestroy();
    }
    /////////////////////////////////////////////////////////////////////////////////

    /////////////////////////////Permissions Management//////////////////////////////

    @Override
    public void onRequestPermissionsResult(int requestCode, String[] permissions,
                                           int[] grantResults) {
        if (requestCode == ACCESS_FINE_LOCATION_REQUEST_CODE) {
            if (grantResults.length == 1) {
                if (grantResults[0] == PackageManager.PERMISSION_GRANTED) {
                    startActivity(new Intent(this,
                            LiveTourActivity.class));
                } else if (grantResults[0] == PackageManager.PERMISSION_DENIED) {
                    new AlertDialog.Builder(this)
                            .setMessage(R.string.gps_permission_denied)
                            .setPositiveButton(android.R.string.ok, null)
                            .show();
                }
            }
        }
    }

    private void checkInternetConnection() {
        ConnectivityManager networkManager = (ConnectivityManager)
                getSystemService(Context.CONNECTIVITY_SERVICE);
        NetworkInfo[] networkInfos = networkManager.getAllNetworkInfo();
        boolean isConnected = false;
        for (NetworkInfo info : networkInfos) {
            if (info.getState() == NetworkInfo.State.CONNECTED) {
                isConnected = true;
                break;
            }
        }
        if (!isConnected) {
            MyPermissions.requestServiceActivation(this, R.string.error_title,
                    R.string.internet_disconnected, INTERNET_ACTIVATION);
        }

    }

    ///////////////////////////////Navigation View///////////////////////////////////////


    @SuppressWarnings("StatementWithEmptyBody")
    @Override
    public boolean onNavigationItemSelected(MenuItem item) {

        switch (item.getItemId()) {
            case R.id.user_signing:
                showSigningMessage();
                break;
            case R.id.menu_preferences:
                showPreferences();
                break;
            case R.id.menu_about:
                showAbout();
                break;
            case R.id.my_sites:
                showMySites();
                break;
            case R.id.sites_to_visit:
                showSitesToVisit();
                break;
            case R.id.live_tour:
                showLiveTour();
                break;
            case R.id.virtual_tour:
                showVirtualTour();
            default:
                break;
        }

        drawer.closeDrawer(GravityCompat.START);
        return true;
    }

    private void showSigningMessage(){
        MyNotifications.createToastNotification(this, R.string.signing_message,
                MyNotifications.LONG_DURATION);
    }

    private void showPreferences() {
        startActivity(new Intent(this, SettingsActivity.class));
    }

    private void showMySites() {
        Intent intent = new Intent(this, MySitesActivity.class);
        intent.putExtra("fieldName", FAVOURITE_LIST);
        intent.putExtra("listName", getResources().getString(R.string.menu_my_sites));
        startActivity(intent);
    }

    private void showSitesToVisit() {
        Intent intent = new Intent(this, MySitesActivity.class);
        intent.putExtra("fieldName", TO_VISIT_LIST);
        intent.putExtra("listName", getResources().getString(R.string.menu_sites_to_visit));
        startActivity(intent);
    }

    private void showAbout() {
        new AlertDialog.Builder(this)
                .setTitle(R.string.menu_about)
                .setMessage(R.string.about_message)
                .setPositiveButton(android.R.string.ok, null)
                .show();
    }

    private void showLiveTour() {
        if (ContextCompat.checkSelfPermission(MainActivity.this,
                Manifest.permission.ACCESS_FINE_LOCATION) ==
                PackageManager.PERMISSION_GRANTED) {
            startActivity(new Intent(this, LiveTourActivity.class));
        } else {
            MyPermissions.requestPermissionsFromActivity(MainActivity.this,
                    new String[]{Manifest.permission.ACCESS_FINE_LOCATION},
                    new int[]{R.string.gps_permission_request},
                    new int[]{ACCESS_FINE_LOCATION_REQUEST_CODE});
        }
    }

    private void showVirtualTour() {
        Intent intent = new Intent(this, VirtualTourActivity.class);
        intent.putExtra("isSite", true);
        startActivity(intent);
    }

    @Override
    public void onBackPressed() {
        if (drawer.isDrawerOpen(GravityCompat.START)) {
            drawer.closeDrawer(GravityCompat.START);
        } else {
            super.onBackPressed();
        }
    }

    //////////////////////////////////ActionBar Options//////////////////////////////////

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case android.R.id.home:
                drawer.openDrawer(GravityCompat.START);
        }
        return super.onOptionsItemSelected(item);
    }

    /////////////////////////////////////////////////////////////////////////////////////

    private void onClickListener() {

        eventAdapter.setItemOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                int position = recyclerView.getChildAdapterPosition(view);
                Site site = eventList.get(position);

                Intent intent = new Intent(MainActivity.this,
                        EventDetailActivity.class);
                intent.putExtra("id", site.getId());
                intent.putExtra("name", site.getName());
                intent.putExtra("type", site.getType().toString());
                intent.putExtra("image", site.getImage());
                intent.putExtra("date", site.getEvent().getDuration());
                intent.putExtra("time", site.getEvent().getTime());
                startActivity(intent);
            }
        });

    }

    private boolean checkSiteType(SiteType type) {
        Set<String> userSelectedTypes = preferences
                .getStringSet("prf_site_type", null);
        return userSelectedTypes != null && userSelectedTypes.contains(type.toString());
    }

    private void checkNewEventsFromNetwork(){

        progressBar.setVisibility(ProgressBar.VISIBLE);
        loadingText.setVisibility(TextView.VISIBLE);

        eventList.clear();
        eventAdapter.swap(eventList);

        databaseReference.addListenerForSingleValueEvent(eventListener);
    }
}